#ifndef CAMERA_PARKING_CLASS_H
#define CAMERA_PARKING_CLASS_H

#include <iostream>
#include <fstream>
#include <thread>

#include <publisher.h>
#include <sys_ctr.h>
#include "ara/log/logging.h"

#include "cam_preprocess.h"
#include "mdc_cam.h"
#include "camera_pool.h"
#include "timer_utils.h"
#include "MbufModule.h"
//#include "wrappers/encoder_wrapper.h"
//#include "wrappers/perception_wrapper.h"

class CameraParking
{
public:
    CameraParking();
    ~CameraParking();
    void init(unsigned int inputImageWidth, unsigned int inputImageHeight, unsigned int outputImageWidth, unsigned int outputImageHeight, unsigned int cropTopOffset, unsigned int cropLeftOffset, unsigned int encoderGopNumber);

protected:
    ara::log::Logger &logger{ara::log::CreateLogger("C_P", "SoftwareMotion Camera Parking Module", ara::log::LogLevel::kDebug)};

private:
    unsigned int inputImageWidth, inputImageHeight, outputImageWidth, outputImageHeight, cropTopOffset, cropLeftOffset, encoderGopNumber;
//    EncoderWrapper _encoderWrapper;
//    PerceptionWrapper _perceptionWrapper;
    
    mbuf::MbufModule mbufManager;
    MDCCamera *camera;
    std::shared_ptr<CameraPool> framePool;
    std::thread pipelineThreadPool[4];
    int encodeImagePublishedCounter[4];

	std::mutex m_writer_mutex1, m_writer_mutex2;

    void sendBanner();
    void *load_file(std::string filepath, size_t &length);
    void threadsFunction(uint8_t id);
    void runPipelines(CamPreprocess &processor, uint8_t id);
};

#endif /* CAMERA_PARKING_CLASS_H */
